@@ -1,6 +1,13 @@
 /*
  * Author: Paul.Russell@rustcorp.com.au and mneuling@radlogic.com.au
  *
+ * (C) 2000-2002 by the netfilter coreteam <coreteam@netfilter.org>:
+ * 		    Paul 'Rusty' Russell <rusty@rustcorp.com.au>
+ * 		    Marc Boucher <marc+nf@mbsi.ca>
+ * 		    James Morris <jmorris@intercode.com.au>
+ * 		    Harald Welte <laforge@gnumonks.org>
+ * 		    Jozsef Kadlecsik <kadlec@blackhole.kfki.hu>
+ *
  * Based on the ipchains code by Paul Russell and Michael Neuling
  *
  *	iptables -- IP firewall administration for kernels with
@@ -37,7 +44,15 @@ main(int argc, char *argv[])
 	ip6tc_handle_t handle = NULL;
 
 	program_name = "ip6tables";
-	program_version = NETFILTER_VERSION;
+	program_version = IPTABLES_VERSION;
+
+	lib_dir = getenv("IP6TABLES_LIB_DIR");
+	if (!lib_dir)
+		lib_dir = IP6T_LIB_DIR;
+
+#ifdef NO_SHARED_LIBS
+	init_extensions();
+#endif
 
 	ret = do_command6(argc, argv, &table, &handle);
 	if (ret)
